@@ -142,15 +142,10 @@ extern int find_new_sub_gids (gid_t *range_start, unsigned long *range_count);
 extern int find_new_sub_uids (uid_t *range_start, unsigned long *range_count);
 #endif				/* ENABLE_SUBIDS */
 
-
-/* get_gid.c */
-extern int get_gid (const char *gidstr, gid_t *gid);
-
 /* getgr_nam_gid.c */
 extern /*@only@*//*@null@*/struct group *getgr_nam_gid (/*@null@*/const char *grname);
 
 /* get_pid.c */
-extern int get_pid (const char *pidstr, pid_t *pid);
 extern int get_pidfd_from_fd(const char *pidfdstr);
 extern int open_pidfd(const char *pidstr);
 
@@ -162,9 +157,6 @@ extern int getrange (const char *range,
 /* gettime.c */
 extern time_t gettime (void);
 
-/* get_uid.c */
-extern int get_uid (const char *uidstr, uid_t *uid);
-
 /* fputsx.c */
 ATTR_ACCESS(write_only, 1, 2)
 extern /*@null@*/char *fgetsx(/*@returned@*/char *restrict, int, FILE *restrict);
