@@ -14,13 +14,15 @@
 
 #ident "$Id$"
 
-#include "alloc.h"
+#include "alloc/calloc.h"
+#include "alloc/malloc.h"
 #include "prototypes.h"
 #include "defines.h"
 #include "commonio.h"
 #include "getdef.h"
-#include "memzero.h"
 #include "sgroupio.h"
+#include "string/memset/memzero.h"
+
 
 /*@null@*/ /*@only@*/struct sgrp *__sgr_dup (const struct sgrp *sgent)
 {
@@ -130,10 +132,9 @@ sgr_free(/*@only@*/struct sgrp *sgent)
 {
 	size_t i;
 	free (sgent->sg_name);
-	if (NULL != sgent->sg_passwd) {
-		strzero (sgent->sg_passwd);
-		free (sgent->sg_passwd);
-	}
+	if (NULL != sgent->sg_passwd)
+		free(strzero(sgent->sg_passwd));
+
 	for (i = 0; NULL != sgent->sg_adm[i]; i++) {
 		free (sgent->sg_adm[i]);
 	}
