@@ -11,10 +11,12 @@
 
 #include <ctype.h>
 
-#ident "$Id$"
-
-#include "prototypes.h"
+#include "atoi/str2i/str2s.h"
 #include "getdate.h"
+#include "prototypes.h"
+#include "string/strchr/stpspn.h"
+#include "string/strcmp/streq.h"
+
 
 /*
  * strtoday() now uses get_date() (borrowed from GNU shellutils)
@@ -41,7 +43,7 @@ long strtoday (const char *str)
 	 * which is not what we expect, unless you're a BOFH :-).
 	 * (useradd sets sp_expire = current date for new lusers)
 	 */
-	if ((NULL == str) || ('\0' == *str)) {
+	if ((NULL == str) || streq(str, "")) {
 		return -1;
 	}
 
@@ -51,10 +53,8 @@ long strtoday (const char *str)
 	if ('-' == *s) {
 		s++;
 	}
-	while (' ' == *s) {
-		s++;
-	}
-	while (isnum && ('\0' != *s)) {
+	s = stpspn(s, " ");
+	while (isnum && !streq(s, "")) {
 		if (!isdigit (*s)) {
 			isnum = false;
 		}
@@ -62,9 +62,8 @@ long strtoday (const char *str)
 	}
 	if (isnum) {
 		long retdate;
-		if (getlong(str, &retdate) == -1) {
+		if (str2sl(&retdate, str) == -1)
 			return -2;
-		}
 		return retdate;
 	}
 
