@@ -26,17 +26,18 @@
 #endif				/* ACCT_TOOLS_SETUID */
 #include <pwd.h>
 
-#include "alloc.h"
-#include "atoi/str2i.h"
+#include "atoi/a2i/a2s.h"
 #include "defines.h"
-#include "memzero.h"
 #include "prototypes.h"
 #include "pwio.h"
 #include "shadowio.h"
 #include "shadowlog.h"
-#include "string/sprintf.h"
+#include "string/memset/memzero.h"
+#include "string/sprintf/snprintf.h"
+#include "string/strcmp/streq.h"
+#include "string/strcpy/strtcpy.h"
+#include "string/strdup/xstrdup.h"
 #include "string/strftime.h"
-#include "string/strtcpy.h"
 #include "time/day_to_str.h"
 /*@-exitarg@*/
 #include "exitcodes.h"
@@ -171,17 +172,13 @@ static int new_fields (void)
 
 	SNPRINTF(buf, "%ld", mindays);
 	change_field (buf, sizeof buf, _("Minimum Password Age"));
-	if (   (str2sl(&mindays, buf) == -1)
-	    || (mindays < -1)) {
+	if (a2sl(&mindays, buf, NULL, 0, -1, LONG_MAX) == -1)
 		return 0;
-	}
 
 	SNPRINTF(buf, "%ld", maxdays);
 	change_field (buf, sizeof buf, _("Maximum Password Age"));
-	if (   (str2sl(&maxdays, buf) == -1)
-	    || (maxdays < -1)) {
+	if (a2sl(&maxdays, buf, NULL, 0, -1, LONG_MAX) == -1)
 		return 0;
-	}
 
 	if (-1 == lstchgdate || lstchgdate > LONG_MAX / DAY)
 		strcpy(buf, "-1");
@@ -190,7 +187,7 @@ static int new_fields (void)
 
 	change_field (buf, sizeof buf, _("Last Password Change (YYYY-MM-DD)"));
 
-	if (strcmp (buf, "-1") == 0) {
+	if (streq(buf, "-1")) {
 		lstchgdate = -1;
 	} else {
 		lstchgdate = strtoday (buf);
@@ -201,17 +198,13 @@ static int new_fields (void)
 
 	SNPRINTF(buf, "%ld", warndays);
 	change_field (buf, sizeof buf, _("Password Expiration Warning"));
-	if (   (str2sl(&warndays, buf) == -1)
-	    || (warndays < -1)) {
+	if (a2sl(&warndays, buf, NULL, 0, -1, LONG_MAX) == -1)
 		return 0;
-	}
 
 	SNPRINTF(buf, "%ld", inactdays);
 	change_field (buf, sizeof buf, _("Password Inactive"));
-	if (   (str2sl(&inactdays, buf) == -1)
-	    || (inactdays < -1)) {
+	if (a2sl(&inactdays, buf, NULL, 0, -1, LONG_MAX) == -1)
 		return 0;
-	}
 
 	if (-1 == expdate || LONG_MAX / DAY < expdate)
 		strcpy(buf, "-1");
@@ -221,7 +214,7 @@ static int new_fields (void)
 	change_field (buf, sizeof buf,
 	              _("Account Expiration Date (YYYY-MM-DD)"));
 
-	if (strcmp (buf, "-1") == 0) {
+	if (streq(buf, "-1")) {
 		expdate = -1;
 	} else {
 		expdate = strtoday (buf);
@@ -250,12 +243,16 @@ print_day_as_date(long day)
 		return;
 	}
 
-	if (gmtime_r(&date, &tm) == NULL) {
-		(void) printf ("time_t: %lu\n", (unsigned long)date);
+	if (localtime_r(&date, &tm) == NULL) {
+		puts(_("future"));
+		return;
+	}
+
+	if (STRFTIME(buf, iflg ? "%F" : "%b %d, %Y", &tm) == 0) {
+		puts(_("future"));
 		return;
 	}
 
-	STRFTIME(buf, iflg ? "%Y-%m-%d" : "%b %d, %Y", &tm);
 	(void) puts (buf);
 }
 
@@ -397,8 +394,7 @@ static void process_flags (int argc, char **argv)
 			break;
 		case 'I':
 			Iflg = true;
-			if (   (str2sl(&inactdays, optarg) == -1)
-			    || (inactdays < -1)) {
+			if (a2sl(&inactdays, optarg, NULL, 0, -1, LONG_MAX) == -1) {
 				fprintf (stderr,
 				         _("%s: invalid numeric argument '%s'\n"),
 				         Prog, optarg);
@@ -410,8 +406,7 @@ static void process_flags (int argc, char **argv)
 			break;
 		case 'm':
 			mflg = true;
-			if (   (str2sl(&mindays, optarg) == -1)
-			    || (mindays < -1)) {
+			if (a2sl(&mindays, optarg, NULL, 0, -1, LONG_MAX) == -1) {
 				fprintf (stderr,
 				         _("%s: invalid numeric argument '%s'\n"),
 				         Prog, optarg);
@@ -420,8 +415,7 @@ static void process_flags (int argc, char **argv)
 			break;
 		case 'M':
 			Mflg = true;
-			if (   (str2sl(&maxdays, optarg) == -1)
-			    || (maxdays < -1)) {
+			if (a2sl(&maxdays, optarg, NULL, 0, -1, LONG_MAX) == -1) {
 				fprintf (stderr,
 				         _("%s: invalid numeric argument '%s'\n"),
 				         Prog, optarg);
@@ -434,8 +428,7 @@ static void process_flags (int argc, char **argv)
 			break;
 		case 'W':
 			Wflg = true;
-			if (   (str2sl(&warndays, optarg) == -1)
-			    || (warndays < -1)) {
+			if (a2sl(&warndays, optarg, NULL, 0, -1, LONG_MAX) == -1) {
 				fprintf (stderr,
 				         _("%s: invalid numeric argument '%s'\n"),
 				         Prog, optarg);
@@ -574,7 +567,7 @@ static void open_files (bool readonly)
 	/*
 	 * For shadow password files we have to lock the file and read in
 	 * the entries as was done for the password file. The user entries
-	 * does not have to exist in this case; a new entry will be created
+	 * do not have to exist in this case; a new entry will be created
 	 * for this user if one does not exist already.
 	 */
 	if (!readonly) {
