@@ -21,22 +21,24 @@
 #include <sys/types.h>
 
 #include "agetpass.h"
-#include "alloc.h"
+#include "alloc/x/xmalloc.h"
 #include "attr.h"
 #include "defines.h"
+/*@-exitarg@*/
+#include "exitcodes.h"
 #include "groupio.h"
-#include "memzero.h"
 #include "nscd.h"
-#include "sssd.h"
 #include "prototypes.h"
 #ifdef SHADOWGRP
 #include "sgroupio.h"
 #endif
-/*@-exitarg@*/
-#include "exitcodes.h"
 #include "shadowlog.h"
-#include "string/sprintf.h"
-#include "string/strtcpy.h"
+#include "sssd.h"
+#include "string/memset/memzero.h"
+#include "string/sprintf/snprintf.h"
+#include "string/strcmp/streq.h"
+#include "string/strcpy/strtcpy.h"
+#include "string/strdup/xstrdup.h"
 
 
 /*
@@ -172,33 +174,29 @@ static void catch_signals (int killed)
  */
 static bool is_valid_user_list (const char *users)
 {
-	const char *username;
-	char *end;
 	bool is_valid = true;
-	/*@owned@*/char *tmpusers = xstrdup (users);
+	char  *dup, *tmpusers;
 
-	for (username = tmpusers;
-	     (NULL != username) && ('\0' != *username);
-	     username = end) {
-		end = strchr (username, ',');
-		if (NULL != end) {
-			*end = '\0';
-			end++;
-		}
+	tmpusers = dup = xstrdup(users);
+
+	while (NULL != tmpusers && !streq(tmpusers, "")) {
+		const char  *u;
+
+		u = strsep(&tmpusers, ",");
 
 		/*
 		 * This user must exist.
 		 */
 
 		/* local, no need for xgetpwnam */
-		if (getpwnam (username) == NULL) {
+		if (getpwnam(u) == NULL) {
 			fprintf (stderr, _("%s: user '%s' does not exist\n"),
-			         Prog, username);
+			         Prog, u);
 			is_valid = false;
 		}
 	}
 
-	free (tmpusers);
+	free(dup);
 
 	return is_valid;
 }
@@ -687,30 +685,8 @@ static void check_perms (const struct group *gr)
 		}
 	} else
 #endif				/* SHADOWGRP */
-	if (!amroot) {
-		/*
-		 * The policy here for changing a group is that
-		 * 1) you must be root or
-		 * 2) you must be the first listed member of the group.
-		 * The first listed member of a group can do anything to
-		 * that group that the root user can. The rationale for
-		 * this hack is that the FIRST user is probably the most
-		 * important user in this entire group.
-		 *
-		 * This feature enabled by default could be a security
-		 * problem when installed on existing systems where the
-		 * first group member might be just a normal user.
-		 * --marekm
-		 */
-#if !defined(FIRST_MEMBER_IS_ADMIN)
+	if (!amroot)
 		failure();
-#endif
-		if (gr->gr_mem[0] == NULL)
-			failure();
-
-		if (strcmp(gr->gr_mem[0], myname) != 0)
-			failure();
-	}
 }
 
 /*
@@ -810,16 +786,8 @@ static void get_group (struct group *gr)
 
 			sg->sg_mem = dup_list (gr->gr_mem);
 
-			sg->sg_adm = XMALLOC(2, char *);
-#ifdef FIRST_MEMBER_IS_ADMIN
-			if (sg->sg_mem[0]) {
-				sg->sg_adm[0] = xstrdup (sg->sg_mem[0]);
-				sg->sg_adm[1] = NULL;
-			} else
-#endif
-			{
-				sg->sg_adm[0] = NULL;
-			}
+			sg->sg_adm = XMALLOC(1, char *);
+			sg->sg_adm[0] = NULL;
 
 		}
 		if (sgr_close () == 0) {
@@ -876,7 +844,7 @@ static void change_passwd (struct group *gr)
 			exit (1);
 		}
 
-		if (strcmp (pass, cp) == 0) {
+		if (streq(pass, cp)) {
 			erase_pass (cp);
 			break;
 		}
