@@ -17,6 +17,7 @@
 #include <grp.h>
 #include <stdint.h>
 #include <stdio.h>
+#include <string.h>
 #include <strings.h>
 #include <sys/types.h>
 #ifdef ACCT_TOOLS_SETUID
@@ -26,20 +27,25 @@
 #endif				/* USE_PAM */
 #endif				/* ACCT_TOOLS_SETUID */
 
-#include "alloc.h"
+#include "alloc/x/xmalloc.h"
+#include "atoi/getnum.h"
 #include "chkname.h"
 #include "defines.h"
 #include "groupio.h"
-#include "pwio.h"
 #include "nscd.h"
-#include "sssd.h"
 #include "prototypes.h"
+#include "pwio.h"
 #ifdef	SHADOWGRP
 #include "sgroupio.h"
 #endif
 #include "shadowlog.h"
-#include "string/stpecpy.h"
-#include "string/stpeprintf.h"
+#include "sssd.h"
+#include "string/sprintf/stpeprintf.h"
+#include "string/strcmp/streq.h"
+#include "string/strcpy/stpecpy.h"
+#include "string/strdup/xstrdup.h"
+
+
 /*
  * exit status values
  */
@@ -150,7 +156,7 @@ static void new_grent (struct group *grent)
 	if (   pflg
 #ifdef SHADOWGRP
 	    && (   (!is_shadow_grp)
-	        || (strcmp (grent->gr_passwd, SHADOW_PASSWD_STRING) != 0))
+	        || !streq(grent->gr_passwd, SHADOW_PASSWD_STRING))
 #endif
 		) {
 		/* Update the password in group if there is no gshadow
@@ -193,7 +199,8 @@ static void new_sgent (struct sgrp *sgent)
  *
  *	grp_update() updates the new records in the memory databases.
  */
-static void grp_update (void)
+static void
+grp_update(void)
 {
 	struct group grp;
 	const struct group *ogrp;
@@ -217,13 +224,13 @@ static void grp_update (void)
 	new_grent (&grp);
 #ifdef	SHADOWGRP
 	if (   is_shadow_grp
-	    && (pflg || nflg)) {
+	    && (pflg || nflg || user_list)) {
 		osgrp = sgr_locate (group_name);
 		if (NULL != osgrp) {
 			sgrp = *osgrp;
 			new_sgent (&sgrp);
 		} else if (   pflg
-		           && (strcmp (grp.gr_passwd, SHADOW_PASSWD_STRING) == 0)) {
+		           && streq(grp.gr_passwd, SHADOW_PASSWD_STRING)) {
 			static char *empty = NULL;
 			/* If there is a gshadow file with no entries for
 			 * the group, but the group file indicates a
@@ -246,7 +253,7 @@ static void grp_update (void)
 	}
 
 	if (user_list) {
-		char *token;
+		char  *u, *ul;
 
 		if (!aflg) {
 			// requested to replace the existing groups
@@ -257,15 +264,30 @@ static void grp_update (void)
 			if (NULL != grp.gr_mem[0])
 				grp.gr_mem = dup_list (grp.gr_mem);
 		}
+#ifdef	SHADOWGRP
+		if (NULL != osgrp) {
+			if (!aflg) {
+				sgrp.sg_mem = XMALLOC(1, char *);
+				sgrp.sg_mem[0] = NULL;
+			} else {
+				if (NULL != sgrp.sg_mem[0])
+					sgrp.sg_mem = dup_list(sgrp.sg_mem);
+			}
+		}
+#endif				/* SHADOWGRP */
 
-		token = strtok(user_list, ",");
-		while (token) {
-			if (prefix_getpwnam (token) == NULL) {
-				fprintf (stderr, _("Invalid member username %s\n"), token);
+		ul = user_list;
+		while (NULL != (u = strsep(&ul, ","))) {
+			if (prefix_getpwnam(u) == NULL) {
+				fprintf(stderr, _("Invalid member username %s\n"), u);
 				exit (E_GRP_UPDATE);
 			}
-			grp.gr_mem = add_list(grp.gr_mem, token);
-			token = strtok(NULL, ",");
+
+			grp.gr_mem = add_list(grp.gr_mem, u);
+#ifdef	SHADOWGRP
+			if (NULL != osgrp)
+				sgrp.sg_mem = add_list(sgrp.sg_mem, u);
+#endif				/* SHADOWGRP */
 		}
 	}
 
@@ -347,39 +369,33 @@ static void check_new_gid (void)
  *	check_new_name() insures that the new name does not exist already.
  *	You can't have the same name twice, period.
  */
-static void check_new_name (void)
+static void
+check_new_name(void)
 {
 	/*
 	 * Make sure they are actually changing the name.
 	 */
-	if (strcmp (group_name, group_newname) == 0) {
+	if (streq(group_name, group_newname)) {
 		nflg = 0;
 		return;
 	}
 
-	if (is_valid_group_name (group_newname)) {
-
-		/*
-		 * If the entry is found, too bad.
-		 */
-		/* local, no need for xgetgrnam */
-		if (prefix_getgrnam (group_newname) != NULL) {
-			fprintf (stderr,
-			         _("%s: group '%s' already exists\n"),
-			         Prog, group_newname);
-			exit (E_NAME_IN_USE);
-		}
-		return;
+	if (!is_valid_group_name(group_newname)) {
+		fprintf(stderr,
+			_("%s: invalid group name '%s'\n"),
+			Prog, group_newname);
+		exit(E_BAD_ARG);
 	}
 
-	/*
-	 * All invalid group names land here.
-	 */
+	/* local, no need for xgetgrnam */
+	if (prefix_getgrnam(group_newname) != NULL) {
+		fprintf(stderr,
+			_("%s: group '%s' already exists\n"),
+			Prog, group_newname);
+		exit(E_NAME_IN_USE);
+	}
 
-	fprintf (stderr,
-	         _("%s: invalid group name '%s'\n"),
-	         Prog, group_newname);
-	exit (E_BAD_ARG);
+	return;
 }
 
 /*
@@ -487,7 +503,7 @@ static void close_files (void)
 
 #ifdef	SHADOWGRP
 	if (   is_shadow_grp
-	    && (pflg || nflg)) {
+	    && (pflg || nflg || user_list)) {
 		if (sgr_close () == 0) {
 			fprintf (stderr,
 			         _("%s: failure while writing changes to %s\n"),
@@ -619,7 +635,7 @@ static void prepare_failure_reports (void)
 	add_cleanup (cleanup_report_mod_group, &info_group);
 #ifdef	SHADOWGRP
 	if (   is_shadow_grp
-	    && (pflg || nflg)) {
+	    && (pflg || nflg || user_list)) {
 		add_cleanup (cleanup_report_mod_gshadow, &info_gshadow);
 	}
 #endif
@@ -646,7 +662,7 @@ static void lock_files (void)
 
 #ifdef	SHADOWGRP
 	if (   is_shadow_grp
-	    && (pflg || nflg)) {
+	    && (pflg || nflg || user_list)) {
 		if (sgr_lock () == 0) {
 			fprintf (stderr,
 			         _("%s: cannot lock %s; try again later.\n"),
@@ -684,7 +700,7 @@ static void open_files (void)
 
 #ifdef	SHADOWGRP
 	if (   is_shadow_grp
-	    && (pflg || nflg)) {
+	    && (pflg || nflg || user_list)) {
 		if (sgr_open (O_CREAT | O_RDWR) == 0) {
 			fprintf (stderr,
 			         _("%s: cannot open %s\n"),
