@@ -18,14 +18,17 @@
 #include <assert.h>
 
 #include "agetpass.h"
-#include "alloc.h"
+#include "alloc/x/xmalloc.h"
+#include "chkname.h"
 #include "defines.h"
-#include "getdef.h"
-#include "prototypes.h"
 /*@-exitarg@*/
 #include "exitcodes.h"
+#include "getdef.h"
+#include "prototypes.h"
 #include "shadowlog.h"
-#include "string/sprintf.h"
+#include "string/sprintf/snprintf.h"
+#include "string/strcmp/streq.h"
+#include "string/strdup/xstrdup.h"
 
 
 /*
@@ -72,7 +75,7 @@ static bool ingroup(const char *name, struct group *gr)
 
 	look = gr->gr_mem;
 	while (*look && notfound)
-		notfound = strcmp (*look++, name);
+		notfound = !streq(*look++, name);
 
 	return !notfound;
 }
@@ -147,7 +150,7 @@ static void check_perms (const struct group *grp,
 		spw_free (spwd);
 	}
 
-	if ((pwd->pw_passwd[0] == '\0') && (grp->gr_passwd[0] != '\0')) {
+	if (streq(pwd->pw_passwd, "") && !streq(grp->gr_passwd, "")) {
 		needspasswd = true;
 	}
 
@@ -185,8 +188,8 @@ static void check_perms (const struct group *grp,
 			goto failure;
 		}
 
-		if (grp->gr_passwd[0] == '\0' ||
-		    strcmp (cpasswd, grp->gr_passwd) != 0) {
+		if (streq(grp->gr_passwd, "") ||
+		    !streq(grp->gr_passwd, cpasswd)) {
 #ifdef WITH_AUDIT
 			SNPRINTF(audit_buf, "authentication new-gid=%lu",
 			         (unsigned long) grp->gr_gid);
@@ -399,7 +402,7 @@ int main (int argc, char **argv)
 
 	/*
 	 * Save my name for error messages and save my real gid in case of
-	 * errors. If there is an error i have to exec a new login shell for
+	 * errors. If there is an error, I have to exec a new login shell for
 	 * the user since her old shell won't have fork'd to create the
 	 * process. Skip over the program name to the next command line
 	 * argument.
@@ -426,7 +429,7 @@ int main (int argc, char **argv)
 	 * injecting arbitrary strings into our stderr/stdout, as this can
 	 * be an exploit vector.
 	 */
-	is_newgrp = (strcmp (Basename (argv[0]), "newgrp") == 0);
+	is_newgrp = streq(Basename (argv[0]), "newgrp");
 	Prog = is_newgrp ? "newgrp" : "sg";
 
 	log_set_progname(Prog);
@@ -459,7 +462,7 @@ int main (int argc, char **argv)
 	 * for sg causes a command string to be executed.
 	 *
 	 * The next argument, if present, must be the new group name. Any
-	 * remaining remaining arguments will be used to execute a command
+	 * remaining arguments will be used to execute a command
 	 * as the named group. If the group name isn't present, I just use
 	 * the login group ID of the current user.
 	 *
@@ -470,8 +473,8 @@ int main (int argc, char **argv)
 	 *      sg [-] groupid [[-c command]
 	 */
 	if (   (argc > 0)
-	    && (   (strcmp (argv[0], "-")  == 0)
-	        || (strcmp (argv[0], "-l") == 0))) {
+	    && (   streq(argv[0], "-")
+	        || streq(argv[0], "-l"))) {
 		argc--;
 		argv++;
 		initflag = true;
@@ -482,6 +485,12 @@ int main (int argc, char **argv)
 		 * not "newgrp".
 		 */
 		if ((argc > 0) && (argv[0][0] != '-')) {
+			if (!is_valid_group_name (argv[0])) {
+				fprintf (
+					stderr, _("%s: provided group is not a valid group name\n"),
+					Prog);
+				goto failure;
+			}
 			group = argv[0];
 			argc--;
 			argv++;
@@ -497,7 +506,7 @@ int main (int argc, char **argv)
 			 * "sg group -c command" (as in the man page) or
 			 * "sg group command" (as in the usage message).
 			 */
-			if ((argc > 1) && (strcmp (argv[0], "-c") == 0)) {
+			if ((argc > 1) && streq(argv[0], "-c")) {
 				command = argv[1];
 			} else {
 				command = argv[0];
@@ -513,6 +522,12 @@ int main (int argc, char **argv)
 			usage ();
 			goto failure;
 		} else if (argv[0] != NULL) {
+			if (!is_valid_group_name (argv[0])) {
+				fprintf (
+					stderr, _("%s: provided group is not a valid group name\n"),
+					Prog);
+				goto failure;
+			}
 			group = argv[0];
 		} else {
 			/*
@@ -538,9 +553,9 @@ int main (int argc, char **argv)
 
 #ifdef HAVE_SETGROUPS
 	/*
-	 * get the current users groupset. The new group will be added to
+	 * get the current user's groupset. The new group will be added to
 	 * the concurrent groupset if there is room, otherwise you get a
-	 * nasty message but at least your real and effective group id's are
+	 * nasty message but at least your real and effective group ids are
 	 * set.
 	 */
 	/* don't use getgroups(0, 0) - it doesn't work on some systems */
@@ -575,7 +590,7 @@ int main (int argc, char **argv)
 	 * now we put her in the new group. The password file entry for her
 	 * current user id has been gotten. If there was no optional group
 	 * argument she will have her real and effective group id set to the
-	 * set to the value from her password file entry.
+	 * value from her password file entry.
 	 *
 	 * If run as newgrp, or as sg with no command, this process exec's
 	 * an interactive subshell with the effective GID of the new group.
@@ -676,8 +691,8 @@ int main (int argc, char **argv)
 #ifdef HAVE_SETGROUPS
 	/*
 	 * I am going to try to add her new group id to her concurrent group
-	 * set. If the group id is already present i'll just skip this part.
-	 * If the group doesn't fit, i'll complain loudly and skip this
+	 * set. If the group id is already present I'll just skip this part.
+	 * If the group doesn't fit, I'll complain loudly and skip this
 	 * part.
 	 */
 	for (i = 0; i < ngroups; i++) {
@@ -736,7 +751,7 @@ int main (int argc, char **argv)
 	}
 
 	/*
-	 * See if the "-c" flag was used. If it was, i just create a shell
+	 * See if the "-c" flag was used. If it was, I just create a shell
 	 * command for her using the argument that followed the "-c" flag.
 	 */
 	if (cflag) {
@@ -752,7 +767,7 @@ int main (int argc, char **argv)
 	}
 
 	/*
-	 * I have to get the pathname of her login shell. As a favor, i'll
+	 * I have to get the pathname of her login shell. As a favor, I'll
 	 * try her environment for a $SHELL value first, and then try the
 	 * password file entry. Obviously this shouldn't be in the
 	 * restricted command directory since it could be used to leave the
@@ -771,7 +786,7 @@ int main (int argc, char **argv)
 	cp = getenv ("SHELL");
 	if (!initflag && (NULL != cp)) {
 		prog = cp;
-	} else if ((NULL != pwd->pw_shell) && ('\0' != pwd->pw_shell[0])) {
+	} else if ((NULL != pwd->pw_shell) && !streq(pwd->pw_shell, "")) {
 		prog = pwd->pw_shell;
 	} else {
 		prog = SHELL;
@@ -784,7 +799,7 @@ int main (int argc, char **argv)
 	progbase = Basename (prog);
 
 	/*
-	 * Switch back to her home directory if i am doing login
+	 * Switch back to her home directory if I am doing login
 	 * initialization.
 	 */
 	if (initflag) {
