@@ -8,13 +8,21 @@
  */
 
 #include <config.h>
+
 #include <errno.h>
 #include <grp.h>
 #include <pwd.h>
 #include <stdio.h>
+#include <string.h>
 #include <sys/types.h>
+
 #include "defines.h"
 #include "prototypes.h"
+#include "string/strchr/stpspn.h"
+#include "string/strchr/strrspn.h"
+#include "string/strcmp/streq.h"
+#include "string/strtok/stpsep.h"
+
 
 #ifndef SUAUTHFILE
 #define SUAUTHFILE "/etc/suauth"
@@ -37,12 +45,9 @@ static int isgrp (const char *, const char *);
 static int lines = 0;
 
 
-int check_su_auth (const char *actual_id,
-                   const char *wanted_id,
-                   bool su_to_root)
+int
+check_su_auth(const char *actual_id, const char *wanted_id, bool su_to_root)
 {
-	int posn, endline;
-	const char field[] = ":";
 	FILE *authfile_fd;
 	char temp[1024];
 	char *to_users;
@@ -67,34 +72,27 @@ int check_su_auth (const char *actual_id,
 	}
 
 	while (fgets (temp, sizeof (temp), authfile_fd) != NULL) {
-		lines++;
-		endline = strlen(temp) - 1;
+		char  *p;
 
-		if (temp[0] == '\0' || temp[endline] != '\n') {
+		lines++;
+
+		if (stpsep(temp, "\n") == NULL) {
 			SYSLOG ((LOG_ERR,
 				 "%s, line %d: line too long or missing newline",
 				 SUAUTHFILE, lines));
 			continue;
 		}
 
-		while (endline > 0 && (temp[endline - 1] == ' '
-				       || temp[endline - 1] == '\t'
-				       || temp[endline - 1] == '\n'))
-			endline--;
-		temp[endline] = '\0';
+		stpcpy(strrspn(temp, " \t"), "");
 
-		posn = 0;
-		while (temp[posn] == ' ' || temp[posn] == '\t')
-			posn++;
-
-		if (temp[posn] == '\n' || temp[posn] == '#'
-		    || temp[posn] == '\0') {
+		p = stpspn(temp, " \t");
+		if (*p == '#' || streq(p, ""))
 			continue;
-		}
-		if (!(to_users = strtok (temp + posn, field))
-		    || !(from_users = strtok (NULL, field))
-		    || !(action = strtok (NULL, field))
-		    || strtok (NULL, field)) {
+
+		to_users = strsep(&p, ":");
+		from_users = strsep(&p, ":");
+		action = strsep(&p, ":");
+		if (action == NULL || p != NULL) {
 			SYSLOG ((LOG_ERR,
 				 "%s, line %d. Bad number of fields.\n",
 				 SUAUTHFILE, lines));
@@ -105,7 +103,7 @@ int check_su_auth (const char *actual_id,
 			continue;
 		if (!applies (actual_id, from_users))
 			continue;
-		if (!strcmp (action, "DENY")) {
+		if (streq(action, "DENY")) {
 			SYSLOG ((su_to_root ? LOG_WARN : LOG_NOTICE,
 				 "DENIED su from '%s' to '%s' (%s)\n",
 				 actual_id, wanted_id, SUAUTHFILE));
@@ -113,14 +111,14 @@ int check_su_auth (const char *actual_id,
 			       stderr);
 			fclose (authfile_fd);
 			return DENY;
-		} else if (!strcmp (action, "NOPASS")) {
+		} else if (streq(action, "NOPASS")) {
 			SYSLOG ((su_to_root ? LOG_NOTICE : LOG_INFO,
 				 "NO password asked for su from '%s' to '%s' (%s)\n",
 				 actual_id, wanted_id, SUAUTHFILE));
 			fputs (_("Password authentication bypassed.\n"),stderr);
 			fclose (authfile_fd);
 			return NOPWORD;
-		} else if (!strcmp (action, "OWNPASS")) {
+		} else if (streq(action, "OWNPASS")) {
 			SYSLOG ((su_to_root ? LOG_NOTICE : LOG_INFO,
 				 "su from '%s' to '%s': asking for user's own password (%s)\n",
 				 actual_id, wanted_id, SUAUTHFILE));
@@ -138,17 +136,16 @@ int check_su_auth (const char *actual_id,
 	return NOACTION;
 }
 
-static int applies (const char *single, char *list)
+static int
+applies(const char *single, char *list)
 {
-	const char split[] = ", ";
 	char *tok;
 
 	int state = 0;
 
-	for (tok = strtok (list, split); tok != NULL;
-	     tok = strtok (NULL, split)) {
+	while (NULL != (tok = strsep(&list, ", "))) {
 
-		if (!strcmp (tok, "ALL")) {
+		if (streq(tok, "ALL")) {
 			if (state != 0) {
 				SYSLOG ((LOG_ERR,
 					 "%s, line %d: ALL in bad place\n",
@@ -156,7 +153,7 @@ static int applies (const char *single, char *list)
 				return 0;
 			}
 			state = 1;
-		} else if (!strcmp (tok, "EXCEPT")) {
+		} else if (streq(tok, "EXCEPT")) {
 			if (state != 1) {
 				SYSLOG ((LOG_ERR,
 					 "%s, line %d: EXCEPT in bas place\n",
@@ -164,7 +161,7 @@ static int applies (const char *single, char *list)
 				return 0;
 			}
 			state = 2;
-		} else if (!strcmp (tok, "GROUP")) {
+		} else if (streq(tok, "GROUP")) {
 			if ((state != 0) && (state != 2)) {
 				SYSLOG ((LOG_ERR,
 					 "%s, line %d: GROUP in bad place\n",
@@ -175,7 +172,7 @@ static int applies (const char *single, char *list)
 		} else {
 			switch (state) {
 			case 0:	/* No control words yet */
-				if (!strcmp (tok, single))
+				if (streq(tok, single))
 					return 1;
 				break;
 			case 1:	/* An all */
@@ -184,7 +181,7 @@ static int applies (const char *single, char *list)
 					 SUAUTHFILE, lines));
 				return 0;
 			case 2:	/* All except */
-				if (!strcmp (tok, single))
+				if (streq(tok, single))
 					return 0;
 				break;
 			case 3:	/* Group */
